@@ -7,6 +7,8 @@ Household functions.
 # Packages
 import numpy as np
 from ogcore import tax, utils
+import logging
+from ogcore import config
 
 """
 ------------------------------------------------------------------------
@@ -141,7 +143,8 @@ def get_bq(BQ, j, p, method):
     Calculate bequests to each household.
 
     .. math::
-        bq_{j,s,t} = \zeta_{j,s}\frac{BQ_{t}}{\lambda_{j}\omega_{s,t}}
+        \hat{bq}_{j,s,t} = \zeta_{j,s}
+        \frac{\hat{BQ}_{t}}{\lambda_{j}\hat{\omega}_{s,t}} \quad\forall j,s,t
 
     Args:
         BQ (array_like): aggregate bequests
@@ -202,7 +205,8 @@ def get_tr(TR, j, p, method):
     Calculate transfers to each household.
 
     .. math::
-        tr_{j,s,t} = \zeta_{j,s}\frac{TR_{t}}{\lambda_{j}\omega_{s,t}}
+        \hat{tr}_{j,s,t} = \zeta_{j,s}
+        \frac{\hat{TR}_{t}}{\lambda_{j}\hat{\omega}_{s,t}} \quad\forall j,s,t
 
     Args:
         TR (array_like): aggregate transfers
@@ -212,7 +216,7 @@ def get_tr(TR, j, p, method):
             'TPI'
 
     Returns:
-        tr (array_like): transfers received by each household
+        tr (array_like): transfers received by household
 
     """
     if j is not None:
@@ -238,23 +242,71 @@ def get_tr(TR, j, p, method):
     return tr
 
 
-def get_cons(r, w, p_tilde, b, b_splus1, n, bq, net_tax, e, p):
+def get_rm(RM, j, p, method):
     r"""
-    Calculate household consumption.
+    Calculate remittances to each household.
 
     .. math::
-        c_{j,s,t} =  \frac{(1 + r_{t})b_{j,s,t} + w_t e_{j,s} n_{j,s,t}
-        + bq_{j,s,t} + tr_{j,s,t} - T_{j,s,t} -
-        e^{g_y}b_{j,s+1,t+1}}{1 - \tau^{c}_{s,t}}
+        \hat{rm}_{j,s,t} = \eta_{RM,j,s,t}
+        \frac{\hat{RM}_{t}}{\lambda_{j}\hat{\omega}_{s,t}} \quad\forall j,s,t
 
     Args:
-        r (array_like): the real interest rate
+        RM (array_like): aggregate remittances
+        j (int): index of lifetime ability group
+        p (OG-Core Specifications object): model parameters
+        method (str): adjusts calculation dimensions based on 'SS' or
+            'TPI'
+
+    Returns:
+        rm (array_like): remittances received by household
+
+    """
+    if j is not None:
+        if method == "SS":
+            rm = (p.eta_RM[-1, :, j] * RM) / (p.lambdas[j] * p.omega_SS)
+        else:
+            len_T = RM.shape[0]
+            rm = (p.eta_RM[:len_T, :, j] * RM.reshape((len_T, 1))) / (
+                p.lambdas[j] * p.omega[:len_T, :]
+            )
+    else:
+        if method == "SS":
+            rm = (p.eta_RM[-1, :, :] * RM) / (
+                p.lambdas.reshape((1, p.J)) * p.omega_SS.reshape((p.S, 1))
+            )
+        else:
+            len_T = RM.shape[0]
+            rm = (p.eta_RM[:len_T, :, :] * utils.to_timepath_shape(RM)) / (
+                p.lambdas.reshape((1, 1, p.J))
+                * p.omega[:len_T, :].reshape((len_T, p.S, 1))
+            )
+
+    return rm
+
+
+def get_cons(r_p, w, p_tilde, b, b_splus1, n, bq, rm, net_tax, e, p):
+    r"""
+    Calculate household composite consumption.
+
+    .. math::
+        \hat{c}_{j,s,t} &= \biggl[(1 + r_{p,t})\hat{b}_{j,s,t}
+        + \hat{w}_t e_{j,s}n_{j,s,t} + \hat{bq}_{j,s,t} + \hat{rm}_{j,s,t}
+        + \hat{tr}_{j,s,t} + \hat{ubi}_{j,s,t} + \hat{pension}_{j,s,t}
+        - \hat{tax}_{j,s,t} \\
+        &\qquad - \sum_{i=1}^I\left(1 + \tau^c_{i,t}\right)p_{i,t}\hat{c}_{min,i}
+        - e^{g_y}\hat{b}_{j,s+1,t+1}\biggr] / p_t \\
+        &\qquad\qquad\forall j,t \quad\text{and}\quad E+1\leq s\leq E+S
+        \quad\text{where}\quad \hat{b}_{j,E+1,t}=0
+
+    Args:
+        r_p (array_like): the real interest rate
         w (array_like): the real wage rate
         p_tilde (array_like): the ratio of real GDP to nominal GDP
         b (Numpy array): household savings
         b_splus1 (Numpy array): household savings one period ahead
         n (Numpy array): household labor supply
         bq (Numpy array): household bequests received
+        rm (Numpy array): household remittances received
         net_tax (Numpy array): household net taxes paid
         e (Numpy array): effective labor units
         p (OG-Core Specifications object): model parameters
@@ -264,8 +316,14 @@ def get_cons(r, w, p_tilde, b, b_splus1, n, bq, net_tax, e, p):
 
     """
     cons = (
-        (1 + r) * b + w * e * n + bq - b_splus1 * np.exp(p.g_y) - net_tax
+        (1 + r_p) * b
+        + w * e * n
+        + bq
+        + rm
+        - net_tax
+        - b_splus1 * np.exp(p.g_y)
     ) / p_tilde
+
     return cons
 
 
@@ -320,6 +378,7 @@ def FOC_savings(
     b_splus1,
     n,
     bq,
+    rm,
     factor,
     tr,
     ubi,
@@ -353,6 +412,7 @@ def FOC_savings(
         b_splus2 (Numpy array): household savings two periods ahead
         n (Numpy array): household labor supply
         bq (Numpy array): household bequests received
+        rm (Numpy array): household remittances received
         factor (scalar): scaling factor converting model units to dollars
         tr (Numpy array): government transfers to household
         ubi (Numpy array): universal basic income payment
@@ -449,7 +509,7 @@ def FOC_savings(
         etr_params,
         p,
     )
-    cons = get_cons(r, w, p_tilde, b, b_splus1, n, bq, taxes, e, p)
+    cons = get_cons(r, w, p_tilde, b, b_splus1, n, bq, rm, taxes, e, p)
     deriv = (
         (1 + r)
         - (
@@ -506,6 +566,7 @@ def FOC_labor(
     b_splus1,
     n,
     bq,
+    rm,
     factor,
     tr,
     ubi,
@@ -539,6 +600,7 @@ def FOC_labor(
         b_splus1 (Numpy array): household savings one period ahead
         n (Numpy array): household labor supply
         bq (Numpy array): household bequests received
+        rm (Numpy array): household bequests received
         factor (scalar): scaling factor converting model units to dollars
         tr (Numpy array): government transfers to household
         ubi (Numpy array): universal basic income payment
@@ -637,7 +699,7 @@ def FOC_labor(
         etr_params,
         p,
     )
-    cons = get_cons(r, w, p_tilde, b, b_splus1, n, bq, taxes, e, p)
+    cons = get_cons(r, w, p_tilde, b, b_splus1, n, bq, rm, taxes, e, p)
     deriv = (
         1
         - tau_payroll
@@ -704,29 +766,30 @@ def constraint_checker_SS(bssmat, nssmat, cssmat, ltilde):
         Warnings: if constraints are violated, warnings printed
 
     """
-    print("Checking constraints on capital, labor, and consumption.")
+    logging.info("Checking constraints on capital, labor, and consumption.")
 
     if (bssmat < 0).any():
-        print("\tWARNING: There is negative capital stock")
+        logging.info("WARNING: There is negative capital stock")
     flag2 = False
     if (nssmat < 0).any():
-        print(
-            "\tWARNING: Labor supply violates nonnegativity ", "constraints."
+        logging.info(
+            "WARNING: Labor supply violates nonnegativity constraints."
         )
         flag2 = True
     if (nssmat > ltilde).any():
-        print("\tWARNING: Labor supply violates the ltilde constraint.")
+        logging.info("WARNING: Labor supply violates the ltilde constraint.")
         flag2 = True
     if flag2 is False:
-        print(
-            "\tThere were no violations of the constraints on labor",
-            " supply.",
+        logging.info(
+            "There were no violations of the constraints on labor supply.",
         )
     if (cssmat < 0).any():
-        print("\tWARNING: Consumption violates nonnegativity", " constraints.")
+        logging.info(
+            "WARNING: Consumption violates nonnegativity constraints."
+        )
     else:
-        print(
-            "\tThere were no violations of the constraints on", " consumption."
+        logging.info(
+            "There were no violations of the constraints on consumption."
         )
 
 
@@ -750,22 +813,20 @@ def constraint_checker_TPI(b_dist, n_dist, c_dist, t, ltilde):
 
     """
     if (b_dist <= 0).any():
-        print(
-            "\tWARNING: Aggregate capital is less than or equal to ",
-            "zero in period %.f." % t,
+        logging.info(
+            f"WARNING: Aggregate capital is less than or equal to zero in period {t}."
         )
     if (n_dist < 0).any():
-        print(
-            "\tWARNING: Labor supply violates nonnegativity",
-            " constraints in period %.f." % t,
+        logging.info(
+            f"WARNING: Labor supply violates nonnegativity constraints in period {t}."
         )
     if (n_dist > ltilde).any():
-        print(
+        logging.info(
             "\tWARNING: Labor suppy violates the ltilde constraint",
             " in period %.f." % t,
         )
     if (c_dist < 0).any():
-        print(
+        logging.info(
             "\tWARNING: Consumption violates nonnegativity",
             " constraints in period %.f." % t,
         )
