@@ -1,6 +1,7 @@
 import numpy as np
 import os
 import matplotlib.pyplot as plt
+import matplotlib.ticker as mticker
 from mpl_toolkits.mplot3d import Axes3D
 import matplotlib
 from ogcore.constants import (
@@ -47,8 +48,7 @@ def plot_aggregates(
             'levels': plot variables in model units
             'forecast': plots variables in levels relative to baseline
                 economic forecast
-        stationarized (bool): whether used stationarized variables (False
-            only affects pct_diff right now)
+        stationarized (bool): whether used stationarized variables
         num_years_to_plot (integer): number of years to include in plot
         start_year (integer): year to start plot
         forecast_data (array_like): baseline economic forecast series,
@@ -75,23 +75,23 @@ def plot_aggregates(
     if plot_type == "pct_diff" or plot_type == "diff":
         assert reform_tpi is not None
     fig1, ax1 = plt.subplots()
+    if not stationarized:
+        for v in var_list:
+            base_tpi[v] = utils.unstationarize_vars(v, base_tpi, base_params)
+            if reform_tpi:
+                reform_tpi[v] = utils.unstationarize_vars(
+                    v, reform_tpi, reform_params
+                )
     for i, v in enumerate(var_list):
+        assert (
+            v in VAR_LABELS.keys()
+        ), "{} is not in the list of variable labels".format(v)
         if plot_type == "pct_diff":
             if v in ["r_gov", "r", "r_p"]:
                 # Compute just percentage point changes for rates
                 plot_var = reform_tpi[v] - base_tpi[v]
             else:
-                if stationarized:
-                    plot_var = (reform_tpi[v] - base_tpi[v]) / base_tpi[v]
-                else:
-                    pct_changes = utils.pct_change_unstationarized(
-                        base_tpi,
-                        base_params,
-                        reform_tpi,
-                        reform_params,
-                        output_vars=[v],
-                    )
-                    plot_var = pct_changes[v]
+                plot_var = (reform_tpi[v] - base_tpi[v]) / base_tpi[v]
             ylabel = r"Pct. change"
             plt.plot(
                 year_vec,
@@ -175,9 +175,10 @@ def plot_industry_aggregates(
     base_params,
     reform_tpi=None,
     reform_params=None,
-    var_list=["Y_vec"],
+    var_list=["Y_m"],
     ind_names_list=None,
     plot_type="pct_diff",
+    stationarized=True,
     num_years_to_plot=50,
     start_year=DEFAULT_START_YEAR,
     forecast_data=None,
@@ -197,6 +198,7 @@ def plot_industry_aggregates(
         reform_params (OG-Core Specifications class): reform parameters
             object
         var_list (list): names of variable to plot
+
         plot_type (string): type of plot, can be:
             'pct_diff': plots percentage difference between baseline
                 and reform ((reform-base)/base)
@@ -205,6 +207,7 @@ def plot_industry_aggregates(
             'levels': plot variables in model units
             'forecast': plots variables in levels relative to baseline
                 economic forecast
+        stationarized (bool): whether used stationarized variables
         num_years_to_plot (integer): number of years to include in plot
         start_year (integer): year to start plot
         forecast_data (array_like): baseline economic forecast series,
@@ -232,11 +235,22 @@ def plot_industry_aggregates(
         assert base_params.start_year == reform_params.start_year
     year_vec = np.arange(start_year, start_year + num_years_to_plot)
     start_index = start_year - base_params.start_year
+    # Unstationarize variables if needed
+    if not stationarized:
+        for v in var_list:
+            base_tpi[v] = utils.unstationarize_vars(v, base_tpi, base_params)
+            if reform_tpi:
+                reform_tpi[v] = utils.unstationarize_vars(
+                    v, reform_tpi, reform_params
+                )
     # Check that reform included if doing pct_diff or diff plot
     if plot_type == "pct_diff" or plot_type == "diff":
         assert reform_tpi is not None
     fig1, ax1 = plt.subplots()
     for i, v in enumerate(var_list):
+        assert (
+            v in VAR_LABELS.keys()
+        ), "{} is not in the list of variable labels".format(v)
         if len(var_list) == 1:
             var_label = ""
         else:
@@ -342,7 +356,7 @@ def ss_3Dplot(
     base_ss,
     reform_params=None,
     reform_ss=None,
-    var="bssmat_splus1",
+    var="b_sp1",
     plot_type="levels",
     plot_title=None,
     path=None,
@@ -385,7 +399,7 @@ def ss_3Dplot(
         data = (reform_ss[var] - base_ss[var]).T
     elif plot_type == "pct_diff":
         data = ((reform_ss[var] - base_ss[var]) / base_ss[var]).T
-    cmap1 = matplotlib.cm.get_cmap("jet")
+    cmap1 = matplotlib.colormaps.get_cmap("jet")
     X, Y = np.meshgrid(domain, Jgrid)
     fig5, ax5 = plt.subplots(subplot_kw={"projection": "3d"})
     ax5.set_xlabel(r"age-$s$")
@@ -449,6 +463,9 @@ def plot_gdp_ratio(
     start_index = start_year - base_params.start_year
     fig1, ax1 = plt.subplots()
     for i, v in enumerate(var_list):
+        assert (
+            v in ToGDP_LABELS.keys()
+        ), "{} is not in the list of variable labels".format(v)
         if plot_type == "levels":
             plot_var_base = (
                 base_tpi[v][: base_params.T] / base_tpi["Y"][: base_params.T]
@@ -529,7 +546,7 @@ def ability_bar(
     base_params,
     reform_tpi,
     reform_params,
-    var="n_mat",
+    var="n",
     num_years=5,
     start_year=DEFAULT_START_YEAR,
     plot_title=None,
@@ -603,7 +620,7 @@ def ability_bar_ss(
     base_params,
     reform_ss,
     reform_params,
-    var="nssmat",
+    var="n",
     plot_title=None,
     path=None,
 ):
@@ -643,7 +660,7 @@ def ability_bar_ss(
     plt.ylabel(r"Percentage Change in " + VAR_LABELS[var])
     if plot_title:
         plt.title(plot_title, fontsize=15)
-    plt.legend(loc=9, bbox_to_anchor=(0.5, -0.15), ncol=2)
+    # plt.legend(loc=9, bbox_to_anchor=(0.5, -0.15), ncol=2)
     if path:
         fig_path1 = os.path.join(path)
         plt.savefig(fig_path1, bbox_inches="tight", dpi=300)
@@ -658,22 +675,23 @@ def tpi_profiles(
     reform_tpi=None,
     reform_params=None,
     by_j=True,
-    var="n_mat",
+    var="n",
     num_years=5,
     start_year=DEFAULT_START_YEAR,
     plot_title=None,
     path=None,
 ):
     """
-    Plot lifecycle profiles of given variable in the SS.
+    Plot lifecycle profiles of given variable over the transition path
 
     Args:
-        base_ss (dictionary): TPI output from baseline run
+        base_tpi (dictionary): TPI output from baseline run
         base_params (OG-Core Specifications class): baseline parameters
             object
-        reform_ss (dictionary): TPI output from reform run
+        reform_tpi (dictionary): TPI output from reform run
         reform_params (OG-Core Specifications class): reform parameters
             object
+        by_j (bool): if True,then plot separate profiles for each j
         var (string): name of variable to plot
         num_year (integer): number of years to compute changes over
         start_year (integer): year to start plot
@@ -745,7 +763,7 @@ def ss_profiles(
     reform_ss=None,
     reform_params=None,
     by_j=True,
-    var="nssmat",
+    var="n",
     plot_data=None,
     plot_title=None,
     path=None,
@@ -760,6 +778,7 @@ def ss_profiles(
         reform_ss (dictionary): SS output from reform run
         reform_params (OG-Core Specifications class): reform parameters
             object
+        by_j (bool): if True,then plot separate profiles for each j
         var (string): name of variable to plot
         plot_data (array_like): series of data to add to plot
         plot_title (string): title for plot
@@ -983,13 +1002,13 @@ def plot_all(base_output_path, reform_output_path, save_path):
 
     # Pct change in c, n, b, y, etr, mtrx, mtry by ability group over 10 years
     var_list = [
-        "c_path",
-        "n_mat",
-        "bmat_splus1",
-        "etr_path",
-        "mtrx_path",
-        "mtry_path",
-        "y_before_tax_mat",
+        "c",
+        "n",
+        "b_sp1",
+        "etr",
+        "mtrx",
+        "mtry",
+        "before_tax_income",
     ]
     title_list = [
         "consumption",
@@ -1016,12 +1035,12 @@ def plot_all(base_output_path, reform_output_path, save_path):
 
     # lifetime profiles, base vs reform, SS for c, n, b, y - not by j
     var_list = [
-        "cssmat",
-        "nssmat",
-        "bssmat_splus1",
-        "etr_ss",
-        "mtrx_ss",
-        "mtry_ss",
+        "c",
+        "n",
+        "b_sp1",
+        "etr",
+        "mtrx",
+        "mtry",
     ]
     for i, v in enumerate(var_list):
         ss_profiles(
@@ -1067,7 +1086,7 @@ def inequality_plot(
     base_params,
     reform_tpi=None,
     reform_params=None,
-    var="c_path",
+    var="c",
     ineq_measure="gini",
     pctiles=None,
     plot_type="levels",
@@ -1190,14 +1209,17 @@ def inequality_plot(
         plt.title(plot_title, fontsize=15)
     vals = ax1.get_yticks()
     if plot_type == "pct_diff":
-        ax1.set_yticklabels(["{:,.2%}".format(x) for x in vals])
+        ticks_loc = ax1.get_yticks().tolist()
+        ax1.yaxis.set_major_locator(mticker.FixedLocator(ticks_loc))
+        ax1.set_yticklabels(["{:,.2%}".format(x) for x in ticks_loc])
     plt.xlim(
         (
             base_params.start_year - 1,
             base_params.start_year + num_years_to_plot,
         )
     )
-    plt.legend(loc=9, bbox_to_anchor=(0.5, -0.15), ncol=2)
+    if plot_type == "levels":
+        plt.legend(loc=9, bbox_to_anchor=(0.5, -0.15), ncol=2)
     if path:
         fig_path1 = os.path.join(path)
         plt.savefig(fig_path1, bbox_inches="tight", dpi=300)
