@@ -4,7 +4,7 @@ import os
 from ogcore.constants import VAR_LABELS, DEFAULT_START_YEAR
 from ogcore import tax
 from ogcore.utils import save_return_table, Inequality
-from ogcore.utils import pct_change_unstationarized
+from ogcore.utils import unstationarize_vars
 
 cur_path = os.path.split(os.path.abspath(__file__))[0]
 
@@ -64,6 +64,13 @@ def macro_table(
         assert base_params.start_year == reform_params.start_year
     year_vec = np.arange(start_year, start_year + num_years)
     start_index = start_year - base_params.start_year
+    if not stationarized:
+        for v in var_list:
+            base_tpi[v] = unstationarize_vars(v, base_tpi, base_params)
+            if reform_tpi:
+                reform_tpi[v] = unstationarize_vars(
+                    v, reform_tpi, reform_params
+                )
     # Check that reform included if doing pct_diff or diff plot
     if output_type == "pct_diff" or output_type == "diff":
         assert reform_tpi is not None
@@ -76,17 +83,7 @@ def macro_table(
     for i, v in enumerate(var_list):
         if output_type == "pct_diff":
             # multiple by 100 so in percentage points
-            if stationarized:
-                results = ((reform_tpi[v] - base_tpi[v]) / base_tpi[v]) * 100
-            else:
-                pct_changes = pct_change_unstationarized(
-                    base_tpi,
-                    base_params,
-                    reform_tpi,
-                    reform_params,
-                    output_vars=[v],
-                )
-                results = pct_changes[v] * 100
+            results = ((reform_tpi[v] - base_tpi[v]) / base_tpi[v]) * 100
             results_years = results[start_index : start_index + num_years]
             results_overall = (
                 (
@@ -164,7 +161,7 @@ def macro_table(
 def macro_table_SS(
     base_ss,
     reform_ss,
-    var_list=["Yss", "Css", "Kss", "Lss", "rss", "wss"],
+    var_list=["Y", "C", "K", "L", "r", "w"],
     table_format=None,
     path=None,
 ):
@@ -198,8 +195,7 @@ def macro_table_SS(
             diff = ((reform_ss[v] - base_ss[v]) / base_ss[v]) * 100
         else:
             diff = (
-                reform_ss["Dss"] / reform_ss["Yss"]
-                - base_ss["Dss"] / base_ss["Yss"]
+                reform_ss["D"] / reform_ss["Y"] - base_ss["D"] / base_ss["Y"]
             )
         table_dict["% Change (or pp diff)"].append(diff)
         # Make df with dict so can use pandas functions
@@ -216,7 +212,7 @@ def ineq_table(
     base_params,
     reform_ss=None,
     reform_params=None,
-    var_list=["cssmat"],
+    var_list=["c"],
     table_format=None,
     path=None,
 ):
@@ -313,7 +309,7 @@ def gini_table(
     base_params,
     reform_ss=None,
     reform_params=None,
-    var_list=["cssmat"],
+    var_list=["c"],
     table_format=None,
     path=None,
 ):
@@ -425,7 +421,7 @@ def wealth_moments_table(
         "Model": [],
     }
     base_ineq = Inequality(
-        base_ss["bssmat_splus1"],
+        base_ss["b_sp1"],
         base_params.omega_SS,
         base_params.lambdas,
         base_params.S,
@@ -453,11 +449,12 @@ def wealth_moments_table(
     return table
 
 
-def tp_output_dump_table(
+def time_series_table(
     base_params,
     base_tpi,
     reform_params=None,
     reform_tpi=None,
+    stationarized=True,
     table_format=None,
     path=None,
 ):
@@ -472,6 +469,7 @@ def tp_output_dump_table(
         reform_params (OG-Core Specifications class): reform parameters
             object
         reform_tpi (dictionary): TP output from reform run
+        stationarized (bool): whether to report stationarized output
         table_format (string): format to return table in: 'csv', 'tex',
             'excel', 'json', if None, a DataFrame is returned
         path (string): path to save table to
@@ -502,6 +500,14 @@ def tp_output_dump_table(
         "total_tax_revenue",
         "business_tax_revenue",
     ]
+    # unstationarize variables if needed
+    if not stationarized:
+        for v in vars_to_keep:
+            base_tpi[v] = unstationarize_vars(v, base_tpi, base_params)
+            if reform_tpi:
+                reform_tpi[v] = unstationarize_vars(
+                    v, reform_tpi, reform_params
+                )
     base_dict = {k: base_tpi[k] for k in vars_to_keep}
     # update key names
     base_dict_final = dict(
@@ -585,12 +591,12 @@ def dynamic_revenue_decomposition(
            resulting series of tax revenues. Call these series for the
            baseline and reform A and D, respectively.
         2. Create a third revenue series that is computed using the
-           baseline behavior (i.e., `bmat_s` and `n_mat`) and macro
+           baseline behavior (i.e., `b_s` and `n`) and macro
            variables (`tr`, `bq`, `r`, `w`), but with the tax function
            parameter estimates from the reform policy.  Call this
            series B.
         3. Create a fourth revenue series that is computed using the
-           reform behavior (i.e., `bmat_s` and `n_mat`) and tax
+           reform behavior (i.e., `b_s` and `n`) and tax
            functions estimated on the reform tax policy, but
            the macro variables (`tr`, `bq`, `r`, `w`) from the baseline.
            Call this series C.
@@ -646,9 +652,9 @@ def dynamic_revenue_decomposition(
     indiv_liab["A"] = tax.income_tax_liab(
         base_tpi["r_p"][:T],
         base_tpi["w"][:T],
-        base_tpi["bmat_s"],
-        base_tpi["n_mat"][:T, :, :],
-        base_ss["factor_ss"],
+        base_tpi["b_s"],
+        base_tpi["n"][:T, :, :],
+        base_ss["factor"],
         0,
         None,
         "TPI",
@@ -661,9 +667,9 @@ def dynamic_revenue_decomposition(
     indiv_liab["B"] = tax.income_tax_liab(
         base_tpi["r_p"][:T],
         base_tpi["w"][:T],
-        base_tpi["bmat_s"],
-        base_tpi["n_mat"][:T, :, :],
-        base_ss["factor_ss"],
+        base_tpi["b_s"],
+        base_tpi["n"][:T, :, :],
+        base_ss["factor"],
         0,
         None,
         "TPI",
@@ -676,9 +682,9 @@ def dynamic_revenue_decomposition(
     indiv_liab["C"] = tax.income_tax_liab(
         base_tpi["r_p"][:T],
         base_tpi["w"][:T],
-        reform_tpi["bmat_s"],
-        reform_tpi["n_mat"][:T, :, :],
-        base_ss["factor_ss"],
+        reform_tpi["b_s"],
+        reform_tpi["n"][:T, :, :],
+        base_ss["factor"],
         0,
         None,
         "TPI",
@@ -690,9 +696,9 @@ def dynamic_revenue_decomposition(
     indiv_liab["D"] = tax.income_tax_liab(
         reform_tpi["r_p"][:T],
         reform_tpi["w"][:T],
-        reform_tpi["bmat_s"],
-        reform_tpi["n_mat"][:T, :, :],
-        base_ss["factor_ss"],
+        reform_tpi["b_s"],
+        reform_tpi["n"][:T, :, :],
+        base_ss["factor"],
         0,
         None,
         "TPI",
@@ -703,9 +709,9 @@ def dynamic_revenue_decomposition(
     # Business tax revenue from the baseline simulation
     tax_rev_dict["biz"]["A"] = tax.get_biz_tax(
         base_tpi["w"][:T],
-        base_tpi["Y_vec"][:T, :],
-        base_tpi["L_vec"][:T, :],
-        base_tpi["K_vec"][:T, :],
+        base_tpi["Y_m"][:T, :],
+        base_tpi["L_m"][:T, :],
+        base_tpi["K_m"][:T, :],
         base_tpi["p_m"][:T],
         base_params,
         None,
@@ -715,9 +721,9 @@ def dynamic_revenue_decomposition(
     # the reform tax rates
     tax_rev_dict["biz"]["B"] = tax.get_biz_tax(
         base_tpi["w"][:T],
-        base_tpi["Y_vec"][:T, :],
-        base_tpi["L_vec"][:T, :],
-        base_tpi["K_vec"][:T, :],
+        base_tpi["Y_m"][:T, :],
+        base_tpi["L_m"][:T, :],
+        base_tpi["K_m"][:T, :],
         base_tpi["p_m"][:T],
         reform_params,
         None,
@@ -727,9 +733,9 @@ def dynamic_revenue_decomposition(
     # macros with the reform tax rates
     tax_rev_dict["biz"]["C"] = tax.get_biz_tax(
         base_tpi["w"][:T],
-        reform_tpi["Y_vec"][:T, :],
-        reform_tpi["L_vec"][:T, :],
-        reform_tpi["K_vec"][:T, :],
+        reform_tpi["Y_m"][:T, :],
+        reform_tpi["L_m"][:T, :],
+        reform_tpi["K_m"][:T, :],
         reform_tpi["p_m"][:T],
         reform_params,
         None,
@@ -738,9 +744,9 @@ def dynamic_revenue_decomposition(
     # Business tax revenue from the reform
     tax_rev_dict["biz"]["D"] = tax.get_biz_tax(
         reform_tpi["w"][:T],
-        reform_tpi["Y_vec"][:T, :],
-        reform_tpi["L_vec"][:T, :],
-        reform_tpi["K_vec"][:T, :],
+        reform_tpi["Y_m"][:T, :],
+        reform_tpi["L_m"][:T, :],
+        reform_tpi["K_m"][:T, :],
         reform_tpi["p_m"][:T],
         reform_params,
         None,
@@ -764,17 +770,17 @@ def dynamic_revenue_decomposition(
         # Behavior effect
         pct_change2 = (
             (tax_rev_dict[type]["C"] - tax_rev_dict[type]["B"])
-            / tax_rev_dict[type]["B"]
+            / tax_rev_dict[type]["A"]
         ) * 100
         # Macro effect
         pct_change3 = (
             (tax_rev_dict[type]["D"] - tax_rev_dict[type]["C"])
-            / tax_rev_dict[type]["C"]
+            / tax_rev_dict[type]["A"]
         ) * 100
         # Dynamic effect (behavior + macro)
         pct_change4 = (
             (tax_rev_dict[type]["D"] - tax_rev_dict[type]["B"])
-            / tax_rev_dict[type]["B"]
+            / tax_rev_dict[type]["A"]
         ) * 100
         # Total change in tax revenue (rates + behavior + macro)
         pct_change5 = (
@@ -803,7 +809,7 @@ def dynamic_revenue_decomposition(
                     start_index : start_index + num_years
                 ].sum()
             )
-            / tax_rev_dict[type]["B"][
+            / tax_rev_dict[type]["A"][
                 start_index : start_index + num_years
             ].sum()
         ) * 100
@@ -816,7 +822,7 @@ def dynamic_revenue_decomposition(
                     start_index : start_index + num_years
                 ].sum()
             )
-            / tax_rev_dict[type]["C"][
+            / tax_rev_dict[type]["A"][
                 start_index : start_index + num_years
             ].sum()
         ) * 100
@@ -829,7 +835,7 @@ def dynamic_revenue_decomposition(
                     start_index : start_index + num_years
                 ].sum()
             )
-            / tax_rev_dict[type]["B"][
+            / tax_rev_dict[type]["A"][
                 start_index : start_index + num_years
             ].sum()
         ) * 100
