@@ -13,6 +13,7 @@ import pickle
 import urllib3
 import ssl
 import json
+import collections
 
 EPSILON = 1e-10  # tolerance or comparison functions
 
@@ -401,7 +402,7 @@ def save_return_table(table_df, output_type, path, precision=2):
     Args:
         table_df (Pandas DataFrame): table
         output_type (string): specifies the type of file to save
-            table to: 'csv', 'tex', 'excel', 'json'
+            table to: 'csv', 'tex', 'excel', 'json', 'md'
         path (string): specifies path to save file with table to
         precision (integer): number of significant digits to print.
             Defaults to 0.
@@ -433,6 +434,10 @@ def save_return_table(table_df, output_type, path, precision=2):
             table_df.to_csv(path_or_buf=path, index=False, na_rep="")
         elif output_type == "json":
             table_df.to_json(path_or_buf=path, double_precision=precision)
+        elif output_type == "md":
+            table_df.to_markdown(
+                buf=path, index=False, floatfmt=".{}f".format(precision)
+            )
         elif output_type == "excel":
             table_df.to_excel(excel_writer=path, index=False, na_rep="")
         else:
@@ -869,7 +874,7 @@ def avg_by_bin(x, y, weights=None, bins=10, eql_pctl=True):
     return x_binned, y_binned, weights_binned
 
 
-def extrapolate_arrays(param_in, dims=None, item="Parameter Name"):
+def extrapolate_array(param_in, dims=None, item="Parameter Name"):
     """
     Extrapolates input values to fit model dimensions. Using this allows
     users to input smaller dimensional arrays and have the model infer
@@ -1022,6 +1027,53 @@ def extrapolate_arrays(param_in, dims=None, item="Parameter Name"):
     return param_out
 
 
+def extrapolate_nested_list(list_in, dims=(400, 80, 1)):
+    """
+    Function to extrapolate a nested list to a specified size.
+
+    Currently only set up for 3 deep nested lists, but could be
+    generalized to deeper or shallower lists.
+
+    Args:
+        list_in (list): list to extrapolate
+        dims (tuple): dimensions of the output list
+
+    Returns:
+        list_out (list): extrapolated list
+    """
+    T, S, num_params = dims
+    try:
+        list_in = list_in.tolist()  # in case parameters are numpy arrays
+    except AttributeError:  # catches if they are lists already
+        pass
+    assert isinstance(list_in, list), "please give a list"
+
+    def depth(L):
+        return isinstance(L, list) and max(map(depth, L)) + 1
+
+    # for now, just have this work for 3 deep lists since
+    # the only OG-Core use case is for tax function parameters
+    assert depth(list_in) == 3, "please give a list that is three lists deep"
+    assert depth(list_in) == len(
+        dims
+    ), "please make sure the depth of nested list is equal to the length of dims to extrapolate"
+    # Extrapolate along the first dimension
+    if len(list_in) > T + S:
+        list_in = list_in[: T + S]
+    if len(list_in) < T + S:
+        params_to_add = [list_in[-1]] * (T + S - len(list_in))
+        list_in.extend(params_to_add)
+    # Extrapolate along the second dimension
+    for t in range(len(list_in)):
+        if len(list_in[t]) > S:
+            list_in[t] = list_in[t][:S]
+        if len(list_in[t]) < S:
+            params_to_add = [list_in[t][-1]] * (S - len(list_in[t]))
+            list_in[t].extend(params_to_add)
+
+    return list_in
+
+
 class CustomHttpAdapter(requests.adapters.HTTPAdapter):
     """
     The UN Data Portal server doesn't support "RFC 5746 secure renegotiation". This causes and error when the client is using OpenSSL 3, which enforces that standard by default.
@@ -1157,127 +1209,94 @@ def shift_bio_clock(
     return param_out
 
 
-def pct_change_unstationarized(
-    tpi_base,
-    param_base,
-    tpi_reform,
-    param_reform,
-    output_vars=["K", "Y", "C", "L", "r", "w"],
+def unstationarize_vars(
+    var,
+    tpi_vars,
+    params,
 ):
     """
-    This function takes the time paths of variables from the baseline
-    and reform and parameters from the baseline and reform runs and
-    computes percent changes for each variable in the output_vars list.
-    The function first unstationarizes the time paths of the variables
-    and then computes the percent changes.
+    This function takes a model variable and returns the unstationarized
+    value by adjusting for underlying growth in labor productivity
+    and population growth (where necessary).
 
     Args:
-        tpi_base (Numpy array): time path of the output variables from
+        var (string): name of variable to unstationarize
+        tpi_vars (Numpy array): time path of the output variables from
             the baseline run
-        param_base (Specifications object): dictionary of parameters
+        params (Specifications object): parameters
             from the baseline run
-        tpi_reform (Numpy array): time path of the output variables from
-            the reform run
-        param_reform (Specifications object): dictionary of parameters
-            from the reform run
-        output_vars (list): list of variables for which to compute
-            percent changes
 
     Returns:
-        pct_changes (dict): dictionary of percent changes for each
-            variable in output_vars list
+        pct_changes (array_like): unstationarized values for var
     """
     # compute non-stationary variables
-    non_stationary_output = {"base": {}, "reform": {}}
-    pct_changes = {}
-    T = param_base.T
-    for var in output_vars:
-        if var in [
-            "Y",
-            "B",
-            "K",
-            "K_f",
-            "K_d",
-            "C",
-            "I",
-            "K_g",
-            "I_g",
-            "Y_vec",
-            "K_vec",
-            "C_vec",
-            "I_total",
-            "I_d",
-            "BQ",
-            "TR",
-            "total_tax_revenue",
-            "business_tax_revenue",
-            "iit_payroll_tax_revenue",
-            "iit_revenue",
-            "payroll_tax_revenue",
-            "agg_pension_outlays",
-            "bequest_tax_revenue",
-            "wealth_tax_revenue",
-            "cons_tax_revenue",
-            "G",
-            "D",
-            "D_f",
-            "D_d",
-            "UBI_path",
-            "new_borrowing_f",
-            "debt_service_f",
-        ]:
-            non_stationary_output["base"][var] = (
-                tpi_base[var][:T]
-                * np.cumprod(1 + param_base.g_n[:T])
-                * np.exp(param_base.g_y * np.arange(param_base.T))
-            )
-            non_stationary_output["reform"][var] = (
-                tpi_reform[var][:T]
-                * np.cumprod(1 + param_reform.g_n[:T])
-                * np.exp(param_reform.g_y * np.arange(param_reform.T))
-            )
-        elif var in [
-            "L",
-            "L_vec",
-        ]:
-            non_stationary_output["base"][var] = tpi_base[var][
-                :T
-            ] * np.cumprod(1 + param_base.g_n[:T])
-            non_stationary_output["reform"][var] = tpi_reform[var][
-                :T
-            ] * np.cumprod(1 + param_reform.g_n[:T])
-        elif var in [
-            "w",
-            "ubi_path",
-            "tr_path",
-            "bq_path",
-            "bmat_splus1",
-            "bmat_s",
-            "c_path",
-            "y_before_tax_path",
-            "tax_path",
-        ]:
-            non_stationary_output["base"][var] = tpi_base[var][:T] * np.exp(
-                param_base.g_y * np.arange(param_base.T)
-            )
-            non_stationary_output["reform"][var] = tpi_reform[var][
-                :T
-            ] * np.exp(param_reform.g_y * np.arange(param_reform.T))
-        else:
-            non_stationary_output["base"][var] = tpi_base[var][:T]
-            non_stationary_output["reform"][var] = tpi_reform[var][:T]
+    T = params.T
+    pop_growth = np.cumprod(1 + params.g_n[:T])
+    prod_growth = np.exp(params.g_y * np.arange(params.T))
+    if ("_m" in var) or ("_i" in var):
+        pop_growth = pop_growth.reshape(T, 1)
+        prod_growth = prod_growth.reshape(T, 1)
+    if var in [
+        "Y",
+        "B",
+        "K",
+        "K_f",
+        "K_d",
+        "C",
+        "I",
+        "K_g",
+        "I_g",
+        "Y_m",
+        "K_m",
+        "C_i",
+        "I_total",
+        "I_d",
+        "BQ",
+        "TR",
+        "total_tax_revenue",
+        "business_tax_revenue",
+        "iit_payroll_tax_revenue",
+        "iit_revenue",
+        "payroll_tax_revenue",
+        "agg_pension_outlays",
+        "bequest_tax_revenue",
+        "wealth_tax_revenue",
+        "cons_tax_revenue",
+        "G",
+        "D",
+        "D_f",
+        "D_d",
+        "UBI_path",
+        "new_borrowing_f",
+        "debt_service_f",
+    ]:
+        non_stationary_output = tpi_vars[var][:T] * pop_growth * prod_growth
+    elif var in [
+        "L",
+        "L_m",
+    ]:
+        non_stationary_output = tpi_vars[var][:T] * pop_growth
 
-        # calculate percent change
-        pct_changes[var] = (
-            non_stationary_output["reform"][var]
-            / non_stationary_output["base"][var]
-            - 1
-        )
+    elif var in [
+        "w",
+        "ubi",
+        "tr",
+        "bq",
+        "b_sp1",
+        "b_s",
+        "c",
+        "c_i",
+        "before_tax_income",
+        "hh_taxes",
+    ]:
+        non_stationary_output = tpi_vars[var][:T] * prod_growth
+    else:
+        non_stationary_output = tpi_vars[var][:T]
 
-    return pct_changes
+    return non_stationary_output
 
 
-def param_dump_json(p, path=None):
+def params_to_json(p, path=None):
     """
     This function creates a JSON file with the model parameters of the
     format used for the default_parameters.json file.
@@ -1300,13 +1319,25 @@ def param_dump_json(p, path=None):
         else:
             converted_data[key] = val
 
-    # Parameters that need to be turned into annual rates for default_parameters.json
-    # g_y_annual
-    # beta_annual
-    # delta_annual
-    # delta_tau_annual
-    # delta_g_annual
-    # world_int_rate_annual
+    # Parameters that need to be turned into annual rates for
+    # default_parameters.json
+    annual_list = [
+        "g_y_annual",
+        "beta_annual",
+        "delta_annual",
+        "delta_tau_annual",
+        "delta_g_annual",
+        "world_int_rate_annual",
+    ]
+    for v in annual_list:
+        val = getattr(p, v.replace("_annual", ""))
+        annual_value = (1 + val) ** (
+            p.S / ((p.ending_age - p.starting_age))
+        ) - 1
+        if isinstance(annual_value, np.ndarray):
+            converted_data[v] = annual_value.tolist()
+        else:
+            converted_data[v] = annual_value
 
     # Convert to JSON string
     json_str = json.dumps(converted_data, indent=4)
@@ -1316,3 +1347,46 @@ def param_dump_json(p, path=None):
             f.write(json_str)
     else:
         return json_str
+
+
+def json_to_dict(json_text):
+    """
+    Convert specified JSON text into an ordered Python dictionary.
+
+    Parameters
+    ----------
+    json_text: string
+        JSON text.
+
+    Raises
+    ------
+    ValueError:
+        if json_text contains a JSON syntax error.
+
+    Returns
+    -------
+    dictionary: collections.OrderedDict
+        JSON data expressed as an ordered Python dictionary.
+    """
+    try:
+        ordered_dict = json.loads(
+            json_text, object_pairs_hook=collections.OrderedDict
+        )
+    except ValueError as valerr:
+        text_lines = json_text.split("\n")
+        msg = "Text below contains invalid JSON:\n"
+        msg += str(valerr) + "\n"
+        msg += "Above location of the first error may be approximate.\n"
+        msg += "The invalid JSON text is between the lines:\n"
+        bline = (
+            "XXXX----.----1----.----2----.----3----.----4"
+            "----.----5----.----6----.----7"
+        )
+        msg += bline + "\n"
+        linenum = 0
+        for line in text_lines:
+            linenum += 1
+            msg += "{:04d}{}".format(linenum, line) + "\n"
+        msg += bline + "\n"
+        raise ValueError(msg)
+    return ordered_dict
